/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2017, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package com.redhat.eap.qe.elytron.authnctx;

import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jboss.as.controller.client.ModelControllerClient;
import org.jboss.as.controller.client.ModelControllerClientConfiguration;
import org.jboss.dmr.ModelNode;

/**
 * Servlet for calling ':whoami' management operation directly from server (without any internal setting of
 * AuthenticationContext).
 *
 * @author olukas
 */
@WebServlet(urlPatterns = {DirectCallServlet.SERVLET_PATH})
public class DirectCallServlet extends HttpServlet {

    public static final String SERVLET_PATH = "/directCall";

    public static final String PORT = "port";
    public static final String HOSTNAME = "hostname";

    public static final String WRONG_RESULT = "wrong result";
    public static final String EXECUTION_FAILED = "execution failed";

    /**
     * Optional parameters of request: hostname, port.
     *
     * Servlet call ':whoami' operation through ModelControllerClient based on passed parameters. No AuthenticationContext is
     * initialized in this method. It should use AuthenticationContext obtained from application server.
     *
     * It prints username which was used for calling management operation on AuthenticationContext.
     *
     * @param req
     * @param resp
     * @throws ServletException
     * @throws IOException
     */
    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        resp.setContentType("text/plain");
        int port = req.getParameter(PORT) != null ? Integer.parseInt(req.getParameter(PORT)) : 9990;
        String hostname = req.getParameter(HOSTNAME) != null ? req.getParameter(HOSTNAME) : "localhost";
        String toPrint = WRONG_RESULT;

        try (ModelControllerClient client = ModelControllerClient.Factory
                .create(new ModelControllerClientConfiguration.Builder()
                        .setProtocol("remote+http")
                        .setPort(port)
                        .setHostName(hostname)
                        .setConnectionTimeout(10000).build())) {

            ModelNode operation = new ModelNode();
            operation.get("operation").set("whoami");
            try {
                ModelNode executeResult = client.execute(operation);
                ModelNode result = executeResult.get("result");
                if (result != null) {
                    ModelNode identity = result.get("identity");
                    if (identity != null) {
                        ModelNode username = identity.get("username");
                        if (username != null) {
                            toPrint = username.asString();
                        }
                    }
                }
            } catch (IOException ex) {
                toPrint = EXECUTION_FAILED;
            }
        }

        try (PrintWriter writer = resp.getWriter()) {
            writer.write(toPrint);
        }
    }

}
